@@ -81,6 +81,15 @@ class Action:
     def __init__(self, rel):
         self.rel = rel
 
+    def __repr__(self):
+        return "<%s(%r)>" % (type(self).__name__, PkgRelation.str(self.rel))
+
+    def __eq__(self, other):
+        return isinstance(other, type(self)) and self.rel == other.rel
+
+    def json(self):
+        raise NotImplementedError(self.json)
+
 
 class DropEssential(Action):
     """Drop dependency on essential package."""
@@ -103,6 +112,18 @@ class DropMinimumVersion(Action):
         return ("drop-minimum-version", PkgRelation.str(self.rel))
 
 
+class DropTransition(Action):
+    """Drop dependency on dummy transitional package."""
+
+    def __str__(self):
+        return (
+            "Drop dependency on transitional package %s" %
+            PkgRelation.str(self.rel))
+
+    def json(self):
+        return ("drop-transitional", PkgRelation.str(self.rel))
+
+
 class ReplaceTransition(Action):
     """Replace dependency on dummy transitional package."""
 
@@ -121,6 +142,16 @@ class ReplaceTransition(Action):
         return ("inline-transitional", PkgRelation.str(self.rel),
                 [PkgRelation.str(p) for p in self.replacement])
 
+    def __repr__(self):
+        return "<%s(%r, %r)>" % (
+            type(self).__name__, PkgRelation.str(self.rel),
+            [PkgRelation(p) for p in self.replacement])
+
+    def __eq__(self, other):
+        return (
+            isinstance(other, type(self)) and self.rel == other.rel
+            and self.replacement == other.replacement)
+
 
 class DropObsoleteConflict(Action):
     """Drop conflict with obsolete package."""
@@ -263,7 +294,7 @@ def drop_obsolete_depends(
         keep_minimum_versions: bool = False):
     ors = []
     actions: List[Action] = []
-    for pkgrel in entry:
+    for i, pkgrel in enumerate(entry):
         newrel = pkgrel
         replacement = checker.replacement(pkgrel.name)
         if replacement:
@@ -273,6 +304,9 @@ def drop_obsolete_depends(
                     'Unable to replace multi-package %r', replacement)
             else:
                 newrel = parsed_replacement[0]
+                if newrel in entry:
+                    actions.append(DropTransition(pkgrel))
+                    continue
                 actions.append(ReplaceTransition(pkgrel, parsed_replacement))
         elif pkgrel.version is not None and pkgrel.name != 'debhelper':
             compat_version = checker.package_version(pkgrel.name)
@@ -282,19 +316,27 @@ def drop_obsolete_depends(
             if compat_version is not None and depends_obsolete(
                 compat_version, *pkgrel.version
             ):
-                if not keep_minimum_versions:
-                    newrel = PkgRelation.parse(pkgrel.str())[0]
-                    newrel.version = None
-                    actions.append(DropMinimumVersion(pkgrel))
                 # If the package is essential, we don't need to maintain a
                 # dependency on it.
                 if checker.is_essential(pkgrel.name):
                     actions.append(DropEssential(pkgrel))
                     return [], actions
+                if not keep_minimum_versions:
+                    newrel = PkgRelation.parse(pkgrel.str())[0]
+                    newrel.version = None
+                    actions.append(DropMinimumVersion(pkgrel))
         ors.append(newrel)
+    if not actions:
+        return ors, []
+
+    deduped = []
+    for rel in ors:
+        if rel not in deduped:
+            deduped.append(rel)
+
     # TODO: if dropped: Check if any ors are implied by existing other
     # dependencies
-    return ors, actions
+    return deduped, actions
 
 
 def drop_obsolete_conflicts(checker: PackageChecker, entry: List[PkgRelation]):
@@ -400,8 +442,7 @@ def drop_old_binary_relations(
         keep_minimum_depends_versions: bool = False
         ) -> List[Tuple[str, List[Action], str]]:
     ret = []
-    for field in ["Depends", "Breaks", "Suggests", "Recommends",
-                  "Pre-Depends"]:
+    for field in ["Depends", "Suggests", "Recommends", "Pre-Depends"]:
         actions = update_depends(
             binary, field, package_checker,
             keep_minimum_versions=keep_minimum_depends_versions)
@@ -547,12 +588,12 @@ def _scrub_obsolete(
         with ControlEditor(
                 wt.abspath(control_path),
                 allow_reformatting=allow_reformatting) as editor:
-            specific_files.append(control_path)
             package = editor.source["Source"]
             control_actions = drop_old_relations(
                 editor, binary_package_checker, compat_release,
                 upgrade_release,
                 keep_minimum_depends_versions=keep_minimum_depends_versions)
+        specific_files.extend(wt.safe_relpath_files(editor.changed_files))
     except FileNotFoundError as exc:
         if wt.has_filename(os.path.join(debian_path, "debcargo.toml")):
             control_actions = []
