@@ -17,6 +17,7 @@
 
 """Functions for working with watch files."""
 
+from collections import namedtuple
 from dataclasses import dataclass
 import json
 import logging
@@ -29,7 +30,12 @@ from urllib.request import urlopen, Request
 
 from debian.changelog import Changelog
 from debmutate.reformatting import FormattingUnpreservable
-from debmutate.watch import Watch, WatchEditor, WatchFile
+from debmutate.watch import (
+    Watch,
+    WatchEditor,
+    WatchFile,
+    apply_url_mangle,
+)
 
 from . import (
     USER_AGENT,
@@ -37,6 +43,7 @@ from . import (
     min_certainty,
     certainty_to_confidence,
 )
+from .gpg import fetch_keys
 
 
 @dataclass
@@ -48,6 +55,89 @@ class WatchCandidate:
     preference: Optional[int]
 
 
+class KeyRetrievalFailed(Exception):
+
+    def __init__(self, fingerprints):
+        self.fingerprints = fingerprints
+
+
+COMMON_PGPSIGURL_MANGLES = [
+    's/$/.%s/' % ext for ext in ['asc', 'pgp', 'gpg', 'sig', 'sign']]
+
+
+SignatureInfo = namedtuple('SignatureInfo', ['is_valid', 'keys', 'mangles'])
+
+
+def probe_signature(r, *, pgpsigurlmangle=None, mangles=None,
+                    gpg_context=None):
+    """Try to find the signature file for a release.
+    """
+    import gpg.errors
+
+    if gpg_context is None:
+        import gpg
+        gpg_context = gpg.Context(armor=True)
+
+    if mangles is None:
+        mangles = COMMON_PGPSIGURL_MANGLES
+
+    def sig_valid(sig):
+        return sig.status == 0
+
+    if r.pgpsigurl:
+        pgpsigurls = [(pgpsigurlmangle, r.pgpsigurl)]
+    else:
+        pgpsigurls = [
+            (mangle, apply_url_mangle(mangle, r.url))
+            for mangle in mangles]
+    for mangle, pgpsigurl in pgpsigurls:
+        # Try and download signatures from some predictable locations.
+        try:
+            resp = urlopen(pgpsigurl)
+        except urllib.error.HTTPError:
+            continue
+        except TimeoutError:
+            logging.warning('Timeout error retrieving %s', pgpsigurl)
+            continue
+        sig = resp.read()
+        actual = urlopen(r.url).read()
+        try:
+            gr = gpg_context.verify(actual, sig)[1]
+        except gpg.errors.GPGMEError as e:
+            logging.warning(
+                'Error verifying signature %s on %s: %s',
+                pgpsigurl, r.url, e)
+            continue
+        except gpg.errors.BadSignatures as e:
+            if str(e).endswith(': No public key'):
+                if not fetch_keys(
+                        [s.fpr for s in e.result.signatures],
+                        home_dir=gpg_context.home_dir):
+                    logging.warning(
+                        'Unable to retrieve keys: %r',
+                         e.result.signatures)
+                    raise KeyRetrievalFailed(
+                        [s.fpr for s in e.result.signatures]) from e
+                gr = gpg_context.verify(actual, sig)[1]
+            else:
+                raise
+        signatures = gr.signatures
+        is_valid = True
+        needed_keys = set()
+        for sig in signatures:
+            if not sig_valid(sig):
+                logging.warning(
+                    'Signature from %s in %s for %s not valid',
+                     sig.fpr, pgpsigurl, r.url)
+                is_valid = False
+            else:
+                needed_keys.add(sig.fpr)
+        return SignatureInfo(
+            is_valid=is_valid, keys=needed_keys, mangle=mangle)
+    else:
+        return None
+
+
 def candidates_from_setup_py(
         path, good_upstream_versions: Set[str], net_access=False):
     certainty = "likely"
@@ -73,21 +163,13 @@ def candidates_from_setup_py(
     current_version_filenames = None
     if net_access:
         json_url = "https://pypi.python.org/pypi/%s/json" % project
-        headers = {"User-Agent": USER_AGENT}
         logging.info('Getting %s info on pypi (%s)',
                      project, json_url)
-        try:
-            response = urlopen(
-                Request(json_url, headers=headers),
-                timeout=DEFAULT_URLLIB_TIMEOUT
-            )
-        except urllib.error.HTTPError as e:
-            if e.status == 404:
-                logging.warning('unable to find project %s on pypi',
-                                project)
-                return
-            raise
-        pypi_data = json.load(response)
+        pypi_data = _load_json(json_url)
+        if pypi_data is None:
+            logging.warning('unable to find project %s on pypi',
+                            project)
+            return
         if version in pypi_data["releases"]:
             release = pypi_data["releases"][version]
             current_version_filenames = [
@@ -150,15 +232,22 @@ def candidates_from_upstream_metadata(
 
         code = ruamel.yaml.round_trip_load(inp, preserve_quotes=True)
 
-        try:
-            parsed_url = urlparse(code["Repository"])
-        except KeyError:
-            pass
-        else:
-            if parsed_url.hostname == "github.com":
-                yield from guess_github_watch_entry(
-                    parsed_url, good_upstream_versions, net_access=net_access
-                )
+        for field in ["Repository", "X-Download"]:
+            try:
+                parsed_url = urlparse(code[field].split(' ')[0])
+            except KeyError:
+                pass
+            else:
+                if parsed_url.hostname == "github.com":
+                    yield from guess_github_watch_entry(
+                        parsed_url, good_upstream_versions,
+                        net_access=net_access
+                    )
+                if parsed_url.hostname == "launchpad.net":
+                    yield from guess_launchpad_watch_entry(
+                        parsed_url, good_upstream_versions,
+                        net_access=net_access
+                    )
 
         archive = code.get('Archive')
         if archive == 'CRAN':
@@ -171,6 +260,31 @@ def guess_cran_watch_entry(name):
     yield WatchCandidate(w, "cran", certainty="likely", preference=0)
 
 
+def guess_launchpad_watch_entry(
+        parsed_url, good_upstream_versions, net_access=False):
+    from breezy.branch import Branch
+    import re
+
+    if not net_access:
+        return
+    project = parsed_url.path.strip("/").split("/")[0]
+    url = f"https://api.launchpad.net/devel/{project}/releases"
+    entries = []
+    while url:
+        response = _load_json(url)
+        entries.extend(response['entries'])
+        url = response.get('next_collection_link')
+    files = _load_json(entries[-1]['files_collection_link'])
+    assert len(files['entries']) > 0
+    # TODO(jelmer): add
+    filepattern = files['entries'][0]['file_link'].split('/')[-2].replace(
+        entries[-1]['version'], '(.*)')
+    w = Watch(
+        f'https://launchpad.net/{project}/+download',
+        f'https://launchpad.net/{project}/.*/{filepattern}')
+    yield WatchCandidate(w, "launchpad", certainty="certain", preference=0)
+
+
 def guess_github_watch_entry(
         parsed_url, good_upstream_versions, net_access=False):
     from breezy.branch import Branch
@@ -222,10 +336,7 @@ def guess_github_watch_entry(
     yield WatchCandidate(w, "github", certainty="certain", preference=0)
 
 
-def candidates_from_hackage(package, good_upstream_versions, net_access=False):
-    if not net_access:
-        return
-    url = "https://hackage.haskell.org/package/%s/preferred" % package
+def _load_json(url):
     headers = {"User-Agent": USER_AGENT, "Accept": "application/json"}
     try:
         response = urlopen(
@@ -233,9 +344,18 @@ def candidates_from_hackage(package, good_upstream_versions, net_access=False):
         )
     except urllib.error.HTTPError as e:
         if e.status == 404:
-            return
+            return None
         raise
-    versions = json.load(response)
+    return json.load(response)
+
+
+def candidates_from_hackage(package, good_upstream_versions, net_access=False):
+    if not net_access:
+        return
+    url = "https://hackage.haskell.org/package/%s/preferred" % package
+    versions = _load_json(url)
+    if versions is None:
+        return
     for version in versions["normal-version"]:
         if version in good_upstream_versions:
             break
@@ -437,63 +557,72 @@ def main():  # noqa: C901
         logging.info('%s', get_committer(wt))
         return 0
 
-    try:
-        check_clean_tree(wt, wt.basis_tree(), subpath)
-    except WorkspaceDirty:
-        logging.info("%s: Please commit pending changes first.", wt.basedir)
-        return 1
+    with wt.lock_write():
+        try:
+            check_clean_tree(wt, wt.basis_tree(), subpath)
+        except WorkspaceDirty:
+            logging.info("%s: Please commit pending changes first.",
+                         wt.basedir)
+            return 1
+
+        update_changelog = args.update_changelog
+        allow_reformatting = args.allow_reformatting
+        try:
+            cfg = Config.from_workingtree(wt, subpath)
+        except FileNotFoundError:
+            pass
+        else:
+            if update_changelog is None:
+                update_changelog = cfg.update_changelog()
+            if allow_reformatting is None:
+                allow_reformatting = cfg.allow_reformatting()
 
-    update_changelog = args.update_changelog
-    allow_reformatting = args.allow_reformatting
-    try:
-        cfg = Config.from_workingtree(wt, subpath)
-    except FileNotFoundError:
-        pass
-    else:
-        if update_changelog is None:
-            update_changelog = cfg.update_changelog()
         if allow_reformatting is None:
-            allow_reformatting = cfg.allow_reformatting()
+            allow_reformatting = False
 
-    if allow_reformatting is None:
-        allow_reformatting = False
+        good_upstream_versions = set()
+        expected_versions = list(sorted(good_upstream_versions))[-5:]
 
-    good_upstream_versions = set()
-    expected_versions = list(sorted(good_upstream_versions))[-5:]
+        with open('debian/changelog', 'r') as f:
+            cl = Changelog(f)
+            for entry in cl:
+                good_upstream_versions.add(entry.version.upstream_version)
+            package = cl.package
 
-    with open('debian/changelog', 'r') as f:
-        cl = Changelog(f)
-        for entry in cl:
-            good_upstream_versions.add(entry.version.upstream_version)
-        package = cl.package
-
-    try:
-        with WatchEditor(allow_reformatting=allow_reformatting) as updater:
-            if verify_watch_file(updater.watch_file, package,
-                                 expected_versions):
-                report_fatal(
-                    'nothing-to-do',
-                    'Existing watch file has valid entries')
-                return 0
-            fix_watch_issues(updater)
-            if not verify_watch_file(updater.watch_file, package,
+        try:
+            with WatchEditor(allow_reformatting=allow_reformatting) as updater:
+                if verify_watch_file(updater.watch_file, package,
                                      expected_versions):
-                candidates = find_candidates(
-                    '.', good_upstream_versions,
-                    net_access=not args.disable_net_access)
-                updater.watch_file.entries = [candidates[0].watch]
-    except FileNotFoundError:
-        candidates = find_candidates(
-            '.', good_upstream_versions,
-            net_access=not args.disable_net_access)
-        wf = WatchFile()
-        wf.entries.append(candidates[0].watch)
+                    report_fatal(
+                        'nothing-to-do',
+                        'Existing watch file has valid entries')
+                    return 0
+                fix_watch_issues(updater)
+                if not verify_watch_file(updater.watch_file, package,
+                                         expected_versions):
+                    candidates = find_candidates(
+                        '.', good_upstream_versions,
+                        net_access=not args.disable_net_access)
+                    updater.allow_reformatting = True
+                    updater.watch_file.entries = [candidates[0].watch]
+        except FileNotFoundError:
+            candidates = find_candidates(
+                '.', good_upstream_versions,
+                net_access=not args.disable_net_access)
+            wf = WatchFile()
+            if len(candidates) == 0:
+                report_fatal(
+                    'no-download-urls-found',
+                    'No candidates for the watch file were found')
+                return 1
+            wf.entries.append(candidates[0].watch)
 
-        with open('debian/watch', 'w') as f:
-            wf.dump(f)
-    except FormattingUnpreservable as e:
-        report_fatal('formatting-unpreservable', str(e))
-        return 1
+            with open('debian/watch', 'w') as f:
+                wf.dump(f)
+        except FormattingUnpreservable as e:
+            report_fatal('formatting-unpreservable',
+                         "Unable to preserve formatting of %s" % e.path)
+            return 1
 
     if args.verify:
         with WatchEditor() as updater:
@@ -501,7 +630,8 @@ def main():  # noqa: C901
                                      expected_versions):
                 report_fatal(
                     'verification-failed',
-                    'Unable to verify watch entry %r' % entry)
+                    'Unable to verify watch entry: %s'
+                    % updater.watch_file.entries[0])
                 return 1
 
     if os.environ.get("SVP_API") == "1":
